unit Utils;

interface

uses classes, typinfo, sysutils, ColorGrd, graphics, tabnotbk, winprocs, wintypes;

type
  TPolar=Record
    Angle : Double;
    Magnitude : Double;
  end;

Function IfNull(t,i : String) : String;
Function RRight(c : char; s : string) : String;
Function LLeft(c : Char; s : string) : String;
Function RHSIndexOf(Strings : TStrings; Item : String; Level : Longint) : longint;
Function PropertyAsString(Instance: TPersistent; Name : String) : String;
Function GetPropertyAddress(Instance: TPersistent; name : String) : PPropInfo;
Procedure SpanPages(NoteBook : TTabbedNoteBook);
Function GetMessageId(const s : String) : Word;
Procedure CopyStream(FromStream, ToStream : TMemoryStream);
Procedure CopyValue(Reader : TReader; Writer : TWriter);
Procedure RotatePoint(var Point : TPoint; Deg : Double);
Procedure RectToPolar(var Polar : TPolar; Point : TPoint);
Procedure PolarTorect(var Point : TPoint; Polar : TPolar);
Function DegToRad(Deg : Double) : Double;
Function Arccos(X : Double) : Double;
Function ArcSin(X : Double) : Double;
Procedure OffSetPoint(var Point : TPoint; x, y : Integer);
Function Sign(value : double) : integer;
Function Min(a, b : longint) : longint;
Function Max(a, b : longint) : longint;

implementation


type
  TCardinalSet = set of 0..SizeOf(Cardinal) * 8 - 1;

type
  TFileNameBuf = array[0..79] of Char;

Function IfNull;
begin
  if t='' then result:=i else result:=t;
end;

Function LLeft;
begin
  Result:=Copy(s,0,pos(c,s)-1);
end;

Function RRight;
begin
  Result:=Copy(s,pos(c,s)+1,255);
end;

Function RHSIndexOf;
var
  c : longint;
  tabs, s : String;

begin
  Result:=-1;
  Level:=level-1;
  tabs:=copy(#9#9#9#9#9#9#9#9#9#9,1,level);
  For c:=0 to Strings.Count-1 do begin
    S:=Strings[c];
    If (Item=rright('=',s)) and
       (copy(s,1,level)=tabs) and (s[level+1]<>#9) then begin
      Result:=c;
      Exit;
    end;
  end;
end;

Function PropertyAsString;
var
  PropType: PTypeInfo;
  PropInfo : PPropInfo;

  Function Fix(S : String) : String;
  begin { Strip of letters till we get upper case }
    While (UpCase(s[1])<>s[1]) and (length(s)<>0) do
      s:=copy(s,2,255);
    Result:=s;
  end;

  Procedure WriteStr(S : String);
  begin
    If length(result)<>0 then result:=result+' ';
    Result:=Result+S;
  end;

  procedure WritePropPath;
  begin
{    WritePropName(PPropInfo(PropInfo)^.Name);}
  end;

  procedure WriteSet(Value: Cardinal);
  var
    I: Integer;
    BaseType: PTypeInfo;
  begin
    BaseType := GetTypeData(PropType)^.CompType;
{    WriteValue(vaSet);}
    for I := 0 to 15 do
      if I in TCardinalSet(Value) then WriteStr(fix(GetEnumName(BaseType, I)^));
    WriteStr('');
  end;

  procedure WriteOrdProp;
  var
    Value: Longint;
  begin
    Value := GetOrdProp(Instance, PropInfo);
{    if Value <> PPropInfo(PropInfo)^.Default then}
    begin
      WritePropPath;
      case PropType^.Kind of
        tkSet:
          WriteSet(Value);
        tkEnumeration:
          WriteStr(GetEnumName(PropType, Value)^);
      end;
    end;
  end;

begin
  Result:='';
  PropInfo:=GetPropertyAddress(Instance,name);
  If PropInfo=nil then exit;
  if PPropInfo(PropInfo)^.SetProc <> nil then
  begin
    PropType := PPropInfo(PropInfo)^.PropType;
    case PropType^.Kind of
      tkInteger, tkChar, tkEnumeration, tkSet: WriteOrdProp;
    end;
  end;
end;

Function GetPropertyAddress(Instance: TPersistent; name : String) : PPropInfo;
var
  I, Count: Integer;
  PropInfo: PPropInfo;
  PropList: PPropList;
begin
  name:=Uppercase(name);
  Count := GetTypeData(Instance.ClassInfo)^.PropCount;
  if Count > 0 then
  begin
    GetMem(PropList, Count * SizeOf(Pointer));
    try
      GetPropInfos(Instance.ClassInfo, PropList);
      for I := 0 to Count - 1 do
      begin
        PropInfo := PropList^[I];
        if IsStoredProp(Instance, PropInfo) then
          If Uppercase(PropInfo^.Name)=Name then begin
            Result:=PropInfo;
            break;
          end;
      end;
    finally
      FreeMem(PropList, Count * SizeOf(Pointer));
    end;
  end;
end;

Procedure SpanPages;
var
  i, o : integer;
begin
  o:=NoteBook.PageIndex;
  i:=0;
  NoteBook.TabsPerRow:=NoteBook.TabsPerRow-1;
  While i<=NoteBook.Pages.Count do begin
    NoteBook.PageIndex:=i;
    NoteBook.update;
    inc(i);
  end;
  NoteBook.PageIndex:=o;
end;

Function GetMessageId;
var
  Buffer : Array[0..255] of char;
begin
  Result:=RegisterWindowMessage(StrPCopy(Buffer,s));
end;

Procedure CopyStream;
var
  Buffer : ^Byte;
  BufferSize : word;
  Chunk : longint;
begin
  Buffer:=FromStream.Memory;
  Inc(Buffer,FromStream.Position);
  ToStream.Write(Buffer^,FromStream.Size-FromStream.Position);
  exit;
  buffersize:=32768;
  GetMem(Buffer,BufferSize);
  try
    { Now Copy chunks }
    While FromStream.Position<FromStream.Size do begin
      Chunk:=FromStream.Size-FromStream.Position;
      If Chunk>BufferSize then Chunk:=BufferSize;
      FromStream.Read(Buffer,Chunk);
      ToStream.Write(Buffer,Chunk);
    end;
  finally
    FreeMem(Buffer,BufferSize);
  end;
end;

procedure CopyValue;

  procedure CopyList;
  begin
    while not Reader.EndOfList do CopyValue(Reader,Writer);
    Reader.ReadListEnd;
    Writer.WriteListEnd;
  end;

  procedure CopyBytes(Count: Longint);
  var
    Bytes: array[0..255] of Char;
  begin
    while Count > 0 do
      if Count > SizeOf(Bytes) then
      begin
        Reader.Read(Bytes, SizeOf(Bytes));
        Writer.Write(Bytes, SizeOf(Bytes));
        Dec(Count, SizeOf(Bytes));
      end
      else
      begin
        Reader.Read(Bytes, Count);
        Writer.Write(Bytes, Count);
        Count := 0;
      end;
  end;

  procedure CopyBinary;
  var
    Count: Longint;
  begin
    Reader.Read(Count, SizeOf(Count));
    Writer.Write(Count, SizeOf(Count));
    CopyBytes(Count);
  end;

  procedure CopySetBody;
  var
    S : String;
  begin
    Repeat
      S:=Reader.ReadStr;
      Writer.WriteStr(s);
    Until S='';
  end;

var
  Value : TValueType;
begin
  Reader.Read(Value,SizeOf(Value));
  Writer.Write(Value,Sizeof(Value));
  case Value of
    vaNull: begin end;
    vaList: CopyList;
    vaInt8: CopyBytes(1);
    vaInt16: CopyBytes(2);
    vaInt32: CopyBytes(4);
    vaExtended: CopyBytes(SizeOf(Extended));
    vaString, vaIdent: Writer.WriteStr(Reader.ReadStr);
    vaFalse, vaTrue: begin end;
    vaBinary: CopyBinary;
    vaSet: CopySetBody;
  end;
end;

Procedure RotatePoint;
var
  Polar : TPolar;
begin
  RectToPolar(Polar,Point);
  Polar.Angle:=Polar.Angle+DegToRad(Deg);
  PolarToRect(Point,Polar);
end;

Procedure RectToPolar;
begin
  If Point.X=0 then begin
    If Point.Y>0 then
      Polar.Angle:=pi/2
    else If Point.Y<0 then
      Polar.Angle:=-pi/4
    else { eek! 0,0 WHAT angle!! }
      Polar.Angle:=0;
  end else
    Polar.Angle:=abs(ArcTan(Point.Y / Point.X));
  Polar.Magnitude:=sqrt(sqr(longint(Point.X))+sqr(longint(Point.Y)));
  If (Point.X<0) then Polar.Angle:=pi-Polar.Angle;
end;

Procedure PolarTorect;
begin
  Point.Y:=trunc(Polar.Magnitude*(Sin(Polar.Angle))+0.5);
  Point.X:=trunc(Polar.Magnitude*(Cos(Polar.Angle))+0.5);
end;

Function DegToRad;
begin
  Result:=(Deg*pi)/180;
end;

Function ArcCos;
begin
  Result:=ArcTan (sqrt (1-sqr (x)) /x);
end;

Function ArcSin;
begin
  Result:=ArcTan (x/sqrt (1-sqr (x)));
end;

Procedure OffsetPoint;
begin
  inc(Point.X,X);
  inc(point.Y,Y);
end;

Function Sign;
begin
  If Value>0 then result:=1
  else if Value<0 then result:=-1
  else result:=0;
end;

Function Min;
begin
  if A<B then result:=A else Result:=b;
end;

Function Max;
begin
  if A>B then result:=A else Result:=b;
end;

end.


